@@ -30,43 +30,40 @@
 #include "userlist.h"
 #include "../common/outbound.h"
 
-static void       text_entry_class_init     (TextEntryClass *klass);
-static void       text_entry_init           (TextEntry      *entry);
-static void       text_entry_finalize       (GObject        *object);
-static gboolean   text_entry_key_press      (GtkWidget      *widget,
-                                             GdkEventKey    *event,
-                                             gpointer        data);
+static void       text_entry_class_init        (TextEntryClass *klass);
+static void       text_entry_init              (TextEntry      *entry);
+static void       text_entry_finalize          (GObject        *object);
+static gboolean   text_entry_key_press         (GtkWidget      *widget,
+                                                GdkEventKey    *event,
+                                                gpointer        data);
 #ifdef HAVE_LIBSEXY
-static gboolean   text_entry_spell_check    (TextEntry      *entry,
-                                             gchar          *text,
-                                             gpointer        data);
-static void       enable_spellcheck_changed (GConfClient *client,
-		                             guint cnxn_id,
-					     GConfEntry *gconf_entry,
-					     TextEntry *entry);
-/*
-static void       languages_changed 	    (GConfClient *client,
-		                             guint cnxn_id,
-					     GConfEntry *gconf_entry,
-					     TextEntry *entry);
-*/
+static gboolean   text_entry_spell_check       (TextEntry      *entry,
+                                                gchar          *text,
+                                                gpointer        data);
+static void       enable_spellcheck_changed    (GConfClient    *client,
+		                                guint           cnxn_id,
+					        GConfEntry     *gconf_entry,
+					        TextEntry      *entry);
 #endif
-static void       text_entry_activate       (GtkWidget      *widget,
-                                             gpointer        data);
-static void       text_entry_history_up     (GtkEntry       *entry);
-static void       text_entry_history_down   (GtkEntry       *entry);
-static gboolean   text_entry_tab_complete   (GtkEntry       *entry);
-static void       text_entry_populate_popup (GtkEntry       *entry,
-                                             GtkMenu        *menu,
-                                             gpointer        data);
+static void       text_entry_activate          (GtkWidget      *widget,
+                                                gpointer        data);
+static void       text_entry_history_up        (GtkEntry       *entry);
+static void       text_entry_history_down      (GtkEntry       *entry);
+static gboolean   text_entry_tab_complete      (GtkEntry       *entry);
+static void       text_entry_populate_popup    (GtkEntry       *entry,
+                                                GtkMenu        *menu,
+                                                gpointer        data);
+static void       text_entry_selection_changed (GObject        *obj,
+                                                GParamSpec     *pspec,
+                                                gpointer        data);
 
-static gboolean   tab_complete_command      (GtkEntry       *entry);
-static gboolean   tab_complete_nickname     (GtkEntry       *entry,
-                                             int             start);
-static GtkWidget *get_color_icon            (int             c,
-                                             GtkStyle       *style);
-static void       color_code_activate       (GtkMenuItem    *item,
-                                             gpointer        data);
+static gboolean   tab_complete_command         (GtkEntry       *entry);
+static gboolean   tab_complete_nickname        (GtkEntry       *entry,
+                                                int             start);
+static GtkWidget *get_color_icon               (int             c,
+                                                GtkStyle       *style);
+static void       color_code_activate          (GtkMenuItem    *item,
+                                                gpointer        data);
 
 #ifdef HAVE_LIBSEXY
 static SexySpellEntryClass *parent_class = NULL;
@@ -76,6 +73,8 @@ static GtkEntryClass       *parent_class = NULL;
 G_DEFINE_TYPE (TextEntry, text_entry, GTK_TYPE_ENTRY);
 #endif
 
+static gchar *selected_text = NULL;
+
 struct _TextEntryPriv
 {
 	GCompletion    *command_completion;
@@ -108,11 +107,13 @@ text_entry_init (TextEntry *entry)
 	GError *err  = NULL;
 #endif
 
-	g_signal_connect_after (G_OBJECT (entry), "key_press_event", G_CALLBACK (text_entry_key_press),      NULL);
-	g_signal_connect       (G_OBJECT (entry), "activate",        G_CALLBACK (text_entry_activate),       NULL);
-	g_signal_connect       (G_OBJECT (entry), "populate-popup",  G_CALLBACK (text_entry_populate_popup), NULL);
+	g_signal_connect_after (G_OBJECT (entry), "key_press_event",         G_CALLBACK (text_entry_key_press),         NULL);
+	g_signal_connect       (G_OBJECT (entry), "activate",                G_CALLBACK (text_entry_activate),          NULL);
+	g_signal_connect       (G_OBJECT (entry), "populate-popup",          G_CALLBACK (text_entry_populate_popup),    NULL);
+	g_signal_connect       (G_OBJECT (entry), "notify::cursor-position", G_CALLBACK (text_entry_selection_changed), NULL);
+	g_signal_connect       (G_OBJECT (entry), "notify::selection-bound", G_CALLBACK (text_entry_selection_changed), NULL);
 #ifdef HAVE_LIBSEXY
-	g_signal_connect_after (G_OBJECT (entry), "word-check",      G_CALLBACK (text_entry_spell_check),    NULL);
+	g_signal_connect_after (G_OBJECT (entry), "word-check",              G_CALLBACK (text_entry_spell_check),       NULL);
 #endif
 
 	entry->priv = g_new0 (TextEntryPriv, 1);
@@ -165,10 +166,6 @@ text_entry_init (TextEntry *entry)
 
 	gconf_client_notify_add (client, "/apps/xchat/spellcheck/enabled",
 	                         (GConfClientNotifyFunc) enable_spellcheck_changed, entry, NULL, NULL);
-	/*
-	gconf_client_notify_add (client, "/apps/xchat/spellcheck/languages",
-	                         (GConfClientNotifyFunc) languages_changed, entry, NULL, NULL);
-	*/
 
 	g_object_unref (client);
 #endif
@@ -455,6 +452,7 @@ tab_complete_command (GtkEntry *entry)
 		gtk_editable_set_position (GTK_EDITABLE (entry), pos);
 		g_free (npt);
 		g_free (prefix);
+		g_free (new_prefix);
 		return TRUE;
 	} else {
 		/* more than one match - print a list of options
@@ -470,16 +468,15 @@ tab_complete_command (GtkEntry *entry)
 		conversation_panel_print (CONVERSATION_PANEL (gui.conversation_panel), text_entry->priv->current, (guchar *) printtext, TRUE);
 		g_free (printtext);
 
-		npt = NULL;
 		if (new_prefix && strcasecmp (prefix, new_prefix) != 0) {
 			/* insert the new prefix into the entry */
 			npt = g_strdup_printf ("/%s%s", new_prefix, &text[cursor]);
 			gtk_entry_set_text (entry, npt);
 			g_free (npt);
-			gtk_editable_set_position (GTK_EDITABLE (entry), strlen (new_prefix));
+			gtk_editable_set_position (GTK_EDITABLE (entry), strlen (new_prefix) + 1);
 		}
 		g_free (prefix);
-		g_free (npt);
+		g_free (new_prefix);
 		return TRUE;
 	}
 
@@ -652,6 +649,11 @@ text_entry_set_current (TextEntry *entry, struct session *sess)
 		gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 	}
 	entry->priv->current = sess;
+
+	if (selected_text) {
+		gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+		                        selected_text, strlen (selected_text));
+	}
 }
 
 void
@@ -686,42 +688,20 @@ enable_spellcheck_changed (GConfClient *client, guint cnxn_id, GConfEntry *gconf
 		g_slist_free (langs);
 	}
 }
+#endif
 
-/*
- * FIXME : This is done in preferences-page-spellcheck because if we have 2
- * notifications change on the same gonf key, the one from
- * preferences-page-spellcheck is called before this one and so it doesn't
- * work.  That's suck because we can only change languages using gconf if the
- * preference window is opened.
- */
-#if 0
 static void
-languages_changed (GConfClient *client, guint cnxn_id, GConfEntry *gconf_entry, TextEntry *entry)
+text_entry_selection_changed (GObject *obj, GParamSpec *pspec, gpointer data)
 {
-	GError *err = NULL;
-	GSList *new_languages, *old_languages;
+	GtkEditable *editable;
+	gint start, end;
 
-	new_languages = gconf_client_get_list (client, "/apps/xchat/spellcheck/languages", GCONF_VALUE_STRING, NULL);
-
-	if (new_languages != NULL)
-		sexy_spell_entry_set_active_languages (SEXY_SPELL_ENTRY (entry), new_languages, &err);
-
-	if (err) {
-		g_printerr (_("Error in spellchecking configuration: %s\n"), err->message);
-		g_error_free (err);
-
-		old_languages = sexy_spell_entry_get_active_languages (SEXY_SPELL_ENTRY (entry));
-		if (old_languages != NULL) {
-			gconf_client_set_list (client, "/apps/xchat/spellcheck/languages",
-					       GCONF_VALUE_STRING, old_languages, NULL);
-			g_slist_foreach (old_languages, (GFunc) g_free, NULL);
-			g_slist_free (old_languages);
+	editable = GTK_EDITABLE (obj);
+	if (gtk_editable_get_selection_bounds (editable, &start, &end)) {
+		if (start != end) {
+			if (selected_text)
+				g_free (selected_text);
+			selected_text = gtk_editable_get_chars (editable, start, end);
 		}
 	}
-
-	g_slist_foreach (new_languages, (GFunc) g_free, NULL);
-	g_slist_free (new_languages);
 }
-#endif
-
-#endif
